SystemOrganization addCategory: #'System-CommandLine'!
SystemOrganization addCategory: #'System-CommandLine-Tests'!

TestCase subclass: #BasicCodeLoaderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine-Tests'!

!BasicCodeLoaderTest class methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/4/2012 15:54'!
fakeClassTemplate 
	^ 'Object subclass: #FakeCodeLoaded 
	instanceVariableNames: '''' 
	classVariableNames: '''' 
	poolDictionaries: '''' 
	category: ''Tests-System'''! !

!BasicCodeLoaderTest methodsFor: 'tests' stamp: 'CamilloBruni 5/4/2012 12:06'!
testCommandLineHandlerCondition
	| commandLine reference |
	
	commandLine := CommandLineArguments withArguments: {'/non/existing/file.st'}. 
	"the reponsibility is transfered tot the BasicCodeLoader as soon as there is a .st file in the arguments"
	self assert: (BasicCodeLoader isResponsibleFor: commandLine).
		
	[ reference := FileSystem disk workingDirectory / 'codeLoad1.st'.
	  reference writeStreamDo: [ :stream | stream nextPutAll: '42' ].
	  commandLine := CommandLineArguments withArguments: {reference fullName}. 
	  self assert: (BasicCodeLoader isResponsibleFor: commandLine) ]
	ensure: [ reference delete ]
! !

!BasicCodeLoaderTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/4/2012 15:55'!
testInstallSourceFiles
	| codeLoader reference |
	
	reference := FileSystem disk workingDirectory / 'codeLoad1.st'.
	[  reference writeStreamDo: [ :stream | stream nextPutAll: self class fakeClassTemplate; cr ].
	   codeLoader := BasicCodeLoader new.
	   codeLoader loadSourceFiles: {'codeLoad1.st'}.
	   codeLoader installSourceFiles.
	   self assert:  (Smalltalk at: #FakeCodeLoaded ifAbsent: [ nil ]) notNil ]
	ensure: [ 
		Smalltalk 
			at: #FakeCodeLoaded 
			ifPresent: [ :clazz | clazz removeFromSystem ].
		reference delete ]! !

!BasicCodeLoaderTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
testInstallSourceFilesFail
	| codeLoader reference |
	
	codeLoader := BasicCodeLoader new.
	codeLoader loadSourceFiles: {'codeLoad1.st'}.
	"Ensure file is not there"
	(FileSystem disk workingDirectory / 'codeLoad1.st') asFileReference delete. 
	"Trying non existing file"
	self should: [ codeLoader installSourceFiles ] raise: Error.! !

!BasicCodeLoaderTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
testLoadSourceFiles
	| codeLoader |
	
	codeLoader := BasicCodeLoader new.
	
	codeLoader loadSourceFiles: {
		'/not/existing/codeLoad1.st'. 
		'/not/existing/codeLoad2.st'}.
	
	self assert: (codeLoader instVarNamed: 'sourceFiles') notNil.
	self assert: (codeLoader instVarNamed: 'sourceFiles') size = 2.
	self assert: ((codeLoader instVarNamed: 'sourceFiles') allSatisfy: [ :each | each isKindOf: FileReference ]).! !

TestCase subclass: #CommandLineArgumentsTest
	instanceVariableNames: 'commandLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine-Tests'!

!CommandLineArgumentsTest commentStamp: 'AdrienBarreau 2/12/2011 15:55' prior: 0!
That class tests the API given by AbstractUserInput. Since that class is obviously abstract, it can't be instantiated, the tests are running on CommandLine (which inherits from AbstractUserInput)!

!CommandLineArgumentsTest methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 14:10'!
commandLine
	^ commandLine! !

!CommandLineArgumentsTest methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 12:43'!
parameters
	^ #('noOpt1' 'noOpt2.txt' 'noOpt3.avi' '--option1' 'opt11' 'opt12.txt' 'opt13.avi' '--option2' '--option3' 'opt31.st' 'opt32' '--option4')! !

!CommandLineArgumentsTest methodsFor: 'running' stamp: 'CamilloBruni 5/2/2012 12:31'!
setUp
	commandLine := CommandLineArguments withArguments: self parameters.! !

!CommandLineArgumentsTest methodsFor: 'tests' stamp: 'CamilloBruni 5/2/2012 12:34'!
testAllFileTyped
	self assert: (self commandLine allFilesWithExtension: #txt) = #('noOpt2.txt' 'opt12.txt').
	self assert: (self commandLine allFilesWithExtension: #foo) isEmpty.! !

!CommandLineArgumentsTest methodsFor: 'tests' stamp: 'CamilloBruni 5/2/2012 12:51'!
testAllParameters
	self assert: self commandLine arguments equals: self parameters! !

!CommandLineArgumentsTest methodsFor: 'tests' stamp: 'CamilloBruni 5/2/2012 12:50'!
testHasFilesTyped
	self assert: (self commandLine hasFileWithExtension: #txt).
	self deny: (self commandLine hasFileWithExtension: #foo).! !

!CommandLineArgumentsTest methodsFor: 'tests' stamp: 'CamilloBruni 5/2/2012 12:49'!
testHasOption
	self assert: (self commandLine hasOption: #option1).
	self assert: (self commandLine hasOption: #option4).
	self deny: (self commandLine hasOption: #foo).! !

!CommandLineArgumentsTest methodsFor: 'tests' stamp: 'CamilloBruni 5/1/2012 20:50'!
testHasParameters
	self assert: (self commandLine hasArguments)! !

!CommandLineArgumentsTest methodsFor: 'tests' stamp: 'CamilloBruni 5/2/2012 12:50'!
testParameterAt
	self assert: (self commandLine argumentAt: 1) = 'noOpt1'.
	self assert: (self commandLine argumentAt: 4) = '--option1'.
	self deny: (self commandLine argumentAt: 5) = 'foo'.
	self deny: (self commandLine argumentAt: 1) isNil.! !

TestCase subclass: #CommandLineHandlerTest
	instanceVariableNames: 'commandLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine-Tests'!

!CommandLineHandlerTest methodsFor: 'utility' stamp: 'CamilloBruni 5/2/2012 14:01'!
argumentsWith: aCollection
	^ CommandLineArguments withArguments: aCollection! !

!CommandLineHandlerTest methodsFor: 'tests' stamp: 'CamilloBruni 5/2/2012 14:02'!
testResponsibilityDefault
	| args |
	args := self argumentsWith: #('--help').
	self assert: (DefaultCommandLineHandler isResponsibleFor: args).
	self deny: (BasicCodeLoader isResponsibleFor: args).
	self deny: (EvaluateCommandLineHandler isResponsibleFor: args).! !

!CommandLineHandlerTest methodsFor: 'tests' stamp: 'CamilloBruni 5/2/2012 14:04'!
testResponsibilityEval
	| args |
	args := self argumentsWith: #('eval' '1+2').
	self assert: (DefaultCommandLineHandler isResponsibleFor: args).
	self deny: (BasicCodeLoader isResponsibleFor: args).
	self assert: (EvaluateCommandLineHandler isResponsibleFor: args).
	
	args := self argumentsWith: #('-e' '1+2').
	self assert: (DefaultCommandLineHandler isResponsibleFor: args).
	self deny: (BasicCodeLoader isResponsibleFor: args).
	self assert: (EvaluateCommandLineHandler isResponsibleFor: args).
	
	args := self argumentsWith: #('--evaluate' '1+2').
	self assert: (DefaultCommandLineHandler isResponsibleFor: args).
	self deny: (BasicCodeLoader isResponsibleFor: args).
	self assert: (EvaluateCommandLineHandler isResponsibleFor: args).! !

!CommandLineHandlerTest methodsFor: 'tests' stamp: 'CamilloBruni 5/2/2012 14:02'!
testResponsibilitySt
	| args |
	args := self argumentsWith: #('/foo/bar/myScript.st').
	self assert: (DefaultCommandLineHandler isResponsibleFor: args).
	self assert: (BasicCodeLoader isResponsibleFor: args).
	self deny: (EvaluateCommandLineHandler isResponsibleFor: args).
	
	args := self argumentsWith: #('st' '/foo/bar/myScript.st').
	self assert: (DefaultCommandLineHandler isResponsibleFor: args).
	self assert: (BasicCodeLoader isResponsibleFor: args).
	self deny: (EvaluateCommandLineHandler isResponsibleFor: args).! !

!CommandLineHandlerTest methodsFor: 'tests' stamp: 'CamilloBruni 5/2/2012 14:10'!
testSelectHandlersCodeLoader
	| args handlers |
	args := self argumentsWith: #('/foo/bar/myScript.st').
	handlers := CommandLineHandler selectHandlersFor: args.
	self assert: handlers first = BasicCodeLoader.
	
	args := self argumentsWith: #('/foo/bar/myScript.st' '--verbose').
	handlers := CommandLineHandler selectHandlersFor: args.
	self assert: handlers first = BasicCodeLoader.! !

Object subclass: #CommandLineArguments
	instanceVariableNames: 'arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

CommandLineArguments class
	instanceVariableNames: 'singleton'!

!CommandLineArguments commentStamp: '<historical>' prior: 0!
The CommandLineArguments represents the arguments passed to the image.
In the following case,

	$PHARO_VM myImage.image --foo bar
	
`CommandLineArguments default` contains {'--foo'. 'bar'}.!

CommandLineArguments class
	instanceVariableNames: 'singleton'!

!CommandLineArguments class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/2/2012 11:41'!
default
	^ singleton ifNil: [singleton := self new initialize]! !

!CommandLineArguments class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/1/2012 20:00'!
withArguments: aCollection
	^ self basicNew 
		initializeWithArguments: aCollection;
		yourself! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 12:33'!
allFilesWithExtension: anExtension
	^ self arguments select: [ :arg|
		arg endsWith: anExtension ]! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 19:57'!
argumentAt: index
	^ arguments at: index ! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:42'!
arguments
	^ arguments! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/1/2012 20:49'!
commandLineArguments
	"self commandLineArguments"
	
	| documentPath args |
	
	args := OrderedCollection withAll: Smalltalk arguments.
	documentPath := Smalltalk documentPath.
	documentPath isEmptyOrNil
		ifFalse: [ args addFirst: documentPath ].
 	^ args.! !

!CommandLineArguments methodsFor: 'copying' stamp: 'CamilloBruni 5/1/2012 20:07'!
copySubcommand
	"return a new copy of this CommandLine without the first arguments"
	^ self class withArguments: arguments allButFirst! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/1/2012 20:50'!
hasArguments
	^ arguments size > 0! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/2/2012 12:55'!
hasFileWithExtension: aFileExtension
	"return true if the first argument has the given file extension"
	^ self arguments anySatisfy: [ :arg| 
		arg endsWith: aFileExtension].! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 12:49'!
hasOption: aString
	| option |
	option := (aString size = 1
					ifTrue: [ '-']
					ifFalse: [ '--']), aString.
	^ self arguments includes: option ! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/2/2012 14:07'!
includesSubCommand: aName
	self withFirstArgument: [ :arg|
		arg = aName ifTrue: [ ^ true ]].
	^ false! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/1/2012 21:21'!
initialize
	
	| documentPath  |
	
	arguments := OrderedCollection withAll: Smalltalk arguments.
	documentPath := Smalltalk documentPath.
	documentPath isEmptyOrNil
		ifFalse: [ arguments addFirst: documentPath ].! !

!CommandLineArguments methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/1/2012 20:01'!
initializeWithArguments: aCollection
	super initialize.
	arguments := aCollection.! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 4/28/2012 19:54'!
withFirstArgument: aBlock
	self arguments ifEmpty: [ ^ self ].
	^ aBlock value: self arguments first! !

Object subclass: #CommandLineHandler
	instanceVariableNames: 'commandLine stdout'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!CommandLineHandler commentStamp: '<historical>' prior: 0!
A CommandLineHandler is activated by the CommandLine. 

The responsible handler with the highest priority is selected and its instance-side method #activate is invoked.

By default the handlers are selected by their class name. In the following shell invocation the FooHandler is chosen:

	$PATH_TO_VM myImage.image FooHandler

A handler may provide a short name with the class-side #commandName method. If the FooHandler defined #commandName returning 'foo' it would be activated with the following shell invocation:
	
	$PATH_TO_VM myImage.image foo
	
For more sophisticated handler selection the CommandLineHandler should implement the #isResponsibleFor: class-side method. An instance of the current command line options is passed to this method which should then return a boolean.

Between all the responsible handlers the one with the highes #priority is chosen. To change the priority overwrite the class-side accessor.
!

CommandLineHandler subclass: #BasicCodeLoader
	instanceVariableNames: 'sourceFiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!BasicCodeLoader commentStamp: '<historical>' prior: 0!
CodeLoader provides a way to load smalltalk source file from local.

Example: 

CodeLoader new 
	loadSourceFiles: (Array with: 
		(#file:, 
		FileDirectory default fullName, 
		'/useless.st')); 
	installSourceFiles !

!BasicCodeLoader class methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 11:35'!
commandName
	^ 'st'! !

!BasicCodeLoader class methodsFor: 'accessing' stamp: 'CamilloBruni 5/3/2012 14:14'!
description
	^ 'handles .st source files'! !

!BasicCodeLoader class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:31'!
isResponsibleFor: aCommandLine
	"This handler is reponsible only for .st files"
	(aCommandLine hasFileWithExtension: '.st')
		ifTrue: [ ^ true ].
	^ super isResponsibleFor: aCommandLine! !

!BasicCodeLoader methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 14:17'!
activate
	self loadSourceFiles: (self commandLine allFilesWithExtension: 'st').
	self installSourceFiles.
	(self commandLine hasOption: 'quit')
		ifTrue: [ self exitSucess ].! !

!BasicCodeLoader methodsFor: 'private' stamp: 'CamilloBruni 4/28/2012 21:48'!
handleError: error	
	
	"for syntax errors we can used the default action"	"otherwise resignal it"
	(error isKindOf: SyntaxErrorNotification)
		ifTrue: [ error defaultAction ]
		ifFalse: [ error pass ]! !

!BasicCodeLoader methodsFor: 'private' stamp: 'EstebanLorenzano 4/4/2012 10:38'!
handleError: error reference: aReference
	"Print a header before failing on an errro / syntax notification from the the script loaded by the given request"
	|stderr|
	
	"spit out a warning if in headless mode, otherwise a debugger will popup"
	Smalltalk isHeadless 
		ifTrue: [ self inform: 'Errors in script loaded from ', aReference fullName ].
																														
	(error isKindOf: SyntaxErrorNotification)
		"for syntax errors we can used the default action"
		ifTrue: [ error defaultAction ]
		"otherwise resignal it"
		ifFalse: [ error pass ]! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'EstebanLorenzano 4/4/2012 10:39'!
installSourceFile: aReference
	"Install the previously loaded source file"

	aReference readStreamDo: [ :stream | 			
		[ ^ (FileSystemFileStream with: stream contents asString) reset fileIn ]
			on: Error, ParserNotification
			do: [ :e| 	self handleError: e reference: aReference ] ].! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'EstebanLorenzano 4/4/2012 10:31'!
installSourceFiles
	"Install the previously loaded source files"

	sourceFiles ifNil: [ ^self ].
	[ sourceFiles do: [ :reference | self installSourceFile: reference ] ]
	ensure: [ sourceFiles := nil ].
	! !

!BasicCodeLoader methodsFor: 'loading' stamp: 'CamilloBruni 5/1/2012 20:51'!
loadSourceFiles: anArray
	"Load all the source files in the given array."

	sourceFiles := anArray collect: [ :each | 
						(FileSystem disk resolve: each) asFileReference ].
! !

BasicCodeLoader subclass: #EvaluateCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!EvaluateCommandLineHandler commentStamp: '<historical>' prior: 0!
A CommandLineHandler that reads a string from the command line and outputs the evaluated result.!

!EvaluateCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 22:48'!
commandName
	^ 'eval'! !

!EvaluateCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/3/2012 14:14'!
description
	^ 'directly evaluates passed in one line scripts'! !

!EvaluateCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 14:08'!
isResponsibleFor: commandLineArguments
	"directly handle top-level -e and --evaluate options"
	commandLineArguments withFirstArgument: [ :arg| 
		(#('-e' '--evaluate') includes: arg)
			ifTrue: [ ^ true ]].
		
	^ commandLineArguments includesSubCommand: self commandName! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/1/2012 20:07'!
activate
	self evaluate: self arguments first.
	self quit.! !

!EvaluateCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 21:50'!
evaluate: sourceString
		
	[ self stdout print: (Compiler evaluate: sourceString); cr ]
		on: Error, ParserNotification
		do: [ :e| 	self handleError: e  ].! !

!CommandLineHandler class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/2/2012 11:25'!
activateWith: aCommandLine 
	
	^ self new 
		commandLine: (self prepareSubcommand: aCommandLine);
		activate! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2012 17:38'!
allHandlers
	^ self allSubclasses reject: [ :handler| handler isAbstract ]! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:56'!
commandName
	" Overwrite this method to provide a short name for this command handler.
	You may pass the commandName as first argument to the image to select this handler"
	^ self name! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:53'!
description
	"This method should return a short one-line description of the command"
	^ nil! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2012 17:39'!
isAbstract
	^ self = CommandLineHandler! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 14:06'!
isResponsibleFor: aCommandLineArguments
	^ aCommandLineArguments includesSubCommand: self commandName! !

!CommandLineHandler class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/2/2012 11:28'!
prepareSubcommand: commandLineArguments

	"strip the subcommand name from the arguments"
	commandLineArguments withFirstArgument: [ :arg|
		arg = self commandName 
			ifTrue: [ ^ commandLineArguments copySubcommand ]].
	
	"not a subcommand hence we keep the same args"
	^ commandLineArguments! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 01:51'!
priority
	^ 0! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:30'!
selectHandlersFor: aCommandLine

	^ self allHandlers select: [ :handlerClass|
		handlerClass isResponsibleFor: aCommandLine ]! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:33'!
<< aString
	^ self stdout 
		nextPutAll: aString;
		yourself! !

!CommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 4/28/2012 19:46'!
activate
	 self subclassResponsibility! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:43'!
allHandlers 
	^ CommandLineHandler allHandlers ! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:56'!
arguments
	^ commandLine arguments ! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:28'!
commandLine
	^ commandLine! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:28'!
commandLine: aCommandLine 
	commandLine := aCommandLine! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:42'!
commandName
	^ self class commandName! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:52'!
description
	^ self class description! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:15'!
exitFailure 
	^ Smalltalk exitFailure! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:15'!
exitSucess
	^ Smalltalk exitSucess.! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 4/28/2012 19:47'!
initialize
	super initialize.
	self initializeStdout
	! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 4/28/2012 19:47'!
initializeStdout
	"install the line end conversion and initialize the converter"
	FileStream stdout wantsLineEndConversion: true; converter.
	stdout := FileStream stdout.! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/28/2012 21:36'!
quit
	Smalltalk exitSucess.! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/28/2012 19:47'!
stdout
	^ stdout! !

CommandLineHandler subclass: #DefaultCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!DefaultCommandLineHandler commentStamp: '<historical>' prior: 0!
A DefaultCommandLineHandler handles default command line arguments and options.

The DefaultCommandLineHandler is activated before all other handlers. It first checks if another handler is available. If so it will activate the found handler.!

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/3/2012 14:13'!
description
	^ 'responsible for the default options and activating other commands'! !

!DefaultCommandLineHandler class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/2/2012 11:34'!
initialize 
	"hook into the startup list"
	Smalltalk addToStartUpList: self.! !

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 01:27'!
isResponsibleFor: aCommandLine
	^ true! !

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:34'!
priority 
	"low priority"
	^ Float infinity negated! !

!DefaultCommandLineHandler class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/1/2012 21:18'!
startUp: resuming

	"only handle when lauching a new image"
	resuming ifFalse: [ ^ self ].
	Smalltalk addDeferredStartupAction: [ self new activate ]! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/1/2012 21:23'!
activate
	self arguments ifEmpty: [ ^ self ].
	
	^ self handleArgument: self arguments first.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 21:54'!
copyright
	self stdout 
		nextPutAll: Smalltalk licenseString; cr.
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 5/2/2012 00:23'!
error
	self arguments size = 1
		ifTrue:  [
			(self arguments first beginsWith: '-')
				ifTrue:  [ self << 'Invalid option: ']
				ifFalse: [ self << 'Invalid sub command: ']]
		ifFalse: [ self << 'Invalid arguments: "'].
		
	(self << (' ' join: self arguments)) 
		nextPutAll: '"'; cr.
		
	self printHelp.
	
	Smalltalk isInteractive ifFalse: [ ^ self exitFailure ].
	Error signal: 'Invalid arguments'.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 00:24'!
handleArgument: aString
	
	"give priority to subcommands"
	self handleSubcommand == self
		ifFalse: [ ^ self ].
		
	"check for default options"
	aString = '--version'
		ifTrue: [ ^ self version ].
		
	aString = '--help'
		ifTrue: [ ^ self help ].
		
	aString = '--list'
		ifTrue: [ ^ self list ].
		
	aString = '--copyright'
		ifTrue: [ ^ self copyright ].
	
	"none of the previous options matched hence we output an error message"
	self error.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 00:24'!
handleSubcommand
	"check if there is an active subcommand"
	
	| handlers |
	
	handlers := self selectHandlers.
	
	"ignore the default command line handler"
	handlers first = DefaultCommandLineHandler
		ifTrue: [ ^ self ].
		
	^ handlers first activateWith: commandLine.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/29/2012 16:20'!
help
	self printHelp.
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/1/2012 21:24'!
initialize
	super initialize.
	commandLine := CommandLineArguments new! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/3/2012 14:16'!
list
	| handlers shortNames maxShortNameSize |
	self stdout 
		nextPutAll: 'Currently installed Command Line Handlers:'; cr.
		
	handlers := self allHandlers reject: [ :cls| cls = DefaultCommandLineHandler ].
	shortNames := handlers collect: #commandName.
	maxShortNameSize := (shortNames detectMax: [ :name|  name size ]) size.
	
	handlers do: [ :handler|
		self stdout 
			nextPutAll: '    ';
			nextPutAll: (handler commandName 
								padded: #right to: maxShortNameSize with: $ ); 
			nextPutAll: '    ';
			nextPutAll: (handler description ifNil: [ '--']);cr].
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/1/2012 13:38'!
printHelp
	self stdout 
		nextPutAll: 'Usage:'; cr;
		nextPutAll: '    --help       print this help message'; cr;
		nextPutAll: '    --copyright  print the copyrights'; cr;
		nextPutAll: '    --version    print the version for the image and the vm'; cr;
		nextPutAll: '    --list       list a description of all active command line handlers'; cr.! !

!DefaultCommandLineHandler methodsFor: 'private' stamp: 'CamilloBruni 5/1/2012 21:22'!
selectHandlers
	| handlers |
	handlers := CommandLineHandler selectHandlersFor: commandLine.
	handlers := handlers sort: [ :a :b| a priority >= b priority ].
	^ handlers! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 22:00'!
version
	self stdout 
		nextPutAll: 'Image: '; print: SystemVersion current; cr;
		nextPutAll: 'VM:    '; nextPutAll: Smalltalk vm version; cr.
	self quit.! !
DefaultCommandLineHandler initialize!
